﻿# See "Installation" in the ProductManual.pdf file for details.

Import-Module $PSScriptRoot\Resources\Scripts\New-SelfSignedCertificateEx.ps1 -Force
Import-Module "WebAdministration"

function LogMessage {
    param([string]$Message, [string]$LogFile)
    
    ((Get-Date).ToString() + " - " + $Message) >> $LogFile
}
 

function HP-Write-Host($Message, [Parameter(Mandatory = $false)] $ForegroundColor, [Switch]$NoNewline) {
    if (-not($ForegroundColor)) {
        $ForegroundColor = "White"
    }
    if ($NoNewline) {
        Write-Host $Message -ForegroundColor $ForegroundColor -NoNewline
    }
    else {
        Write-Host $Message -ForegroundColor $ForegroundColor
    }

    LogMessage -Message $Message "$PSScriptRoot/Install.log"
}

function GetFQDNForCurrentServer {
    return (Get-WmiObject win32_computersystem).DNSHostName + "." + (Get-WmiObject win32_computersystem).Domain
}

function DisplaySavedParameters() {
    $flag = $true

    $doc = New-Object System.Xml.XmlDocument

    $configPath = "$PSScriptRoot/Install.Parameters.config"

    if (-Not(Test-Path $configPath)) {
        New-Item -Path $configPath -ItemType File
        Set-Content $configPath '<?xml version="1.0" encoding="utf-8"?>
        <parameters/>'
    }

    $doc.Load($configPath)

    $parameters = "Certificate path", "Site name", "Site path", "Http port", "AppPool UserName", "Sql Server", "Sql Database", "Installation mode" , "Server"

    $existParameters = New-Object -TypeName PSObject

    foreach ($parameterName in $parameters) {
        $node = $doc.SelectSingleNode('parameters/setParameter[@name="' + $parameterName + '"]')

        if ($null -ne $node) {
            $attr = $node.Attributes['value']
            if ($null -ne $attr) {
                $value = $attr.Value
                if (($null -ne $value) -and ($value -ne "")) {
                    if ($flag) {
                        HP-Write-Host("Attached Parameters") -ForegroundColor Yellow
                        $flag = $false
                    }
                    $existParameters | Add-Member -Name $parameterName -Value $value -MemberType NoteProperty
                }    
            }
        }
    }

    HP-Write-Host ($existParameters | Format-List | Out-String)

    return -Not($flag)
}

function ReadOptionalParameterFromConfig([string]$parameterName, $default) {
    $doc = New-Object System.Xml.XmlDocument

    $configPath = "$PSScriptRoot/Install.Parameters.config"

    $doc.Load($configPath)

    $parameters = $doc.SelectSingleNode('parameters')

    if ($null -eq $parameters) {
        $parameters = $doc.CreateNode('element', 'parameters', '')
    }


    $node = $doc.SelectSingleNode('parameters/setParameter[@name="' + $parameterName + '"]')

    if (($null -ne $node) -and ($null -ne $node.Attributes['value']) -and ($node.Attributes['value'].Value -ne "")) {
        return $node.Attributes['value'].Value
    }
    else {
        return $default
    }
}

function GetParameter([string]$parameterName, $parameterDescription, $silent, $systemDefault) {
    $doc = New-Object System.Xml.XmlDocument
    $isEmpty = $false

    $configPath = "$PSScriptRoot/Install.Parameters.config"

    $doc.Load($configPath)

    $parameters = $doc.SelectSingleNode('parameters')

    if ($null -eq $parameters) {
        $parameters = $doc.CreateNode('element', 'parameters', '')
    }

    $node = $doc.SelectSingleNode('parameters/setParameter[@name="' + $parameterName + '"]')

    if ($null -eq $node) {
        $isEmpty = $true
        $node = $doc.CreateNode('element', 'setParameter', '')
        $node.SetAttribute('name', $parameterName)
        $node.SetAttribute('value', $systemDefault)
        $node = $parameters.AppendChild($node)
    }

    $value = $node.Attributes['value'].Value

    if ($value -eq "") {
        do {
            HP-Write-Host($parameterDescription + ":")-NoNewline
            $value = Read-Host
            if ($value -ne "") {
                break
            }
        }while ($true)
        $node.Attributes['value'].Value = $value
    }
    elseif (-not($silent) -or $isEmpty) {
        $defaultValue = $value
        HP-Write-Host ("$parameterDescription (default is $value):") 
        $value = Read-Host
        $value = ($defaultValue, $value)[[bool]$value]
        $node.Attributes['value'].Value = $value
    }

    $doc.Save($configPath)
    return $value
}

function ConfigureFarmPropertyBag([string]$propName, [string]$propValue) {
    $web = Get-SPFarm
    $web.Properties[$propName] = $propValue
    $web.Update()
}

function IsNetFrameworkInstalled() {
    $key = Get-Item 'HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v4\Full'

    if ($null -ne $key) {
        $release = $key.GetValue("Release")
    }
     
    return $release -ge 394254 # .NET Framework 4.6.1
}

function IsMSDeployInstalled(){
    $result = $false
    try{
        $tmp = get-childitem -recurse "HKLM:SOFTWARE\Microsoft\IIS Extensions\MSDeploy" | get-itemproperty | where { $_  -match 'InstallPath' }
        $result = $true
	}
    catch{
    
	}
    return $result
    
}

function WaitForTimer($solutionName) {
    $solution = Get-SPSolution | where-object { $_.Name -eq $solutionName }
    if ($null -ne $solution) {
        HP-Write-Host "Waiting to finish solution timer job" -ForegroundColor Yellow
        while ($solution.JobExists -eq $true ) {
            HP-Write-Host "Please wait... either a retraction or deployment is happening" 
            Start-Sleep 3
        }                
        HP-Write-Host "Finished the solution timer job" -ForegroundColor Yellow
    }
}

function CreateAndImportCertificate() {
    $certLocation = 'cert:\localmachine\my'

    $relativeCertPath = GetParameter "Certificate path" "Please enter relativate path to certificate" $overrideSavedParameters "Resources\HWE2013Cert.pfx"

    $certPath = "$PSScriptRoot\$relativeCertPath"

    if (Test-Path $certPath) {
        HP-Write-Host ( "Certificate file $($certPath) is already exists" ) -ForegroundColor Green
        HP-Write-Host "Please enter password for the existing certificate: " -NoNewline
        $certificatePassword = Read-Host -assecurestring
    }
    else {
        HP-Write-Host ( "Certificate file $($certPath) does not exist, creating new one" ) -ForegroundColor Yellow 
        HP-Write-Host "Please enter password for the new certificate: " -ForegroundColor Yellow -NoNewline
        $certificatePassword = Read-Host -assecurestring
        $cn = "CN=" + $server.Replace('-','')

        $notAfter = [DateTime]::Now.AddYears(100)
        New-SelfSignedCertificateEx -Subject $cn -EKU "1.3.6.1.5.5.7.3.1", "1.3.6.1.5.5.7.3.2" -KeyUsage "KeyEncipherment, DigitalSignature" -SAN $server -AllowSMIME -Exportable -Path "$certPath" -Password $certificatePassword -NotAfter $notAfter
        HP-Write-Host ( "Certificate file $($certPath) has been successfully created" ) -ForegroundColor Green
    }

    do {
        try {
            $certificate = Import-PfxCertificate -FilePath $certPath -CertStoreLocation $certLocation -Exportable -Password $certificatePassword
            break
        }
        catch {
            $ex = $_
        
            if ($ex.Exception.HResult -ne -2146233088) {
                throw
            }
            else {
           
                HP-Write-Host "Password is invalid" -ForegroundColor Red 
                HP-Write-Host "Please enter password for the existing certificate: " 
                $certificatePassword = Read-Host -assecurestring
            }
        }
    }while ($true)

    HP-Write-Host ( "Certificate thumbprint: " + $certificate.Thumbprint ) -ForegroundColor Green
    HP-Write-Host

    HP-Write-Host ( "Adding certificate into local machine certification store..." ) -ForegroundColor Yellow
    $rootStore = New-Object System.Security.Cryptography.X509Certificates.X509Store -ArgumentList Root, LocalMachine
    $rootStore.Open("MaxAllowed")
    $rootStore.Add($certificate)
    $rootStore.Close()

    return $certificate
}

function CreateHWESiteIfNoExist($siteName) {

    $siteItemPath = 'IIS:\Sites\' + $siteName

    if (Test-Path $siteItemPath) {
        HP-Write-Host ( "Site $($siteName) is already exists" ) -ForegroundColor Yellow
        $site = Get-Item $siteItemPath
        
        $sitePath = $site.physicalPath
        $appPoolName = $site.applicationPool
        
        HP-Write-Host ( "Site path: $($sitePath)" ) -ForegroundColor Yellow
        HP-Write-Host ( "Application pool: $($appPoolName)" ) -ForegroundColor Yellow
    }
    else {
        HP-Write-Host ( "Creating a new site $($siteName)..." ) -ForegroundColor Yellow

        $siteItemPath = 'IIS:\Sites\' + $siteName
        $sitePath = GetParameter "Site path" "Please enter physical path for a site" $overrideSavedParameters "c:\inetpub\hwe2013"

        if (Test-Path $sitePath) {
            throw "Path $($sitePath) is alredy exists"
        }
        else {
            New-Item -ItemType directory -Path $sitePath
        }

        $httpPort = GetParameter "Http port" "Please enter a port number for a site" $overrideSavedParameters "8187"

        New-Item $siteItemPath -physicalPath $sitePath -bindings @(@{protocol = "http"; bindingInformation = ":" + $httpPort + ":" + $server }) -Force
        HP-Write-Host "Site $($siteName) has been created" -ForegroundColor Yellow

        $appPoolName = $siteName + 'AppPool'
        $appPoolPath = 'IIS:\AppPools\' + $appPoolName

        if (Test-Path $appPoolPath) {
            HP-Write-Host "Application pool $($appPoolName) is already exists" -ForegroundColor Yellow
        }
        else {
            New-Item $appPoolPath
            HP-Write-Host "Application pool $($appPoolName) has been created" -ForegroundColor Yellow

            $appPoolUserName = GetParameter "AppPool UserName" "Please enter an account name for the application pool" $overrideSavedParameters

            HP-Write-Host "Please enter password for this account: " -ForegroundColor Yellow -NoNewline
            $appPoolUserPassword = Read-Host -assecurestring
            $appPoolUserPasswordDecoded = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($appPoolUserPassword))
            Set-ItemProperty $appPoolPath -name processModel -value @{userName = "$($appPoolUserName)"; password = "$($appPoolUserPasswordDecoded)"; identitytype = 3 }
            HP-Write-Host "Application pool identity has been set to $($appPoolUserName)" -ForegroundColor Green

            $appPool = Get-Item $appPoolPath
            $appPool.Stop()
            $appPool.Start()

            HP-Write-Host ( "Setting properties on $($siteItemPath)..." ) -ForegroundColor Yellow
            Set-ItemProperty $siteItemPath -name applicationPool -value $appPoolName
            
            HP-Write-Host ( "Adding permissions for application pool..." ) -ForegroundColor Yellow

            $appPoolSid = $appPool.applicationPoolSid
            $identifier = New-Object System.Security.Principal.SecurityIdentifier $appPoolSid
            $user = $identifier.Translate([System.Security.Principal.NTAccount])

            $accessRule = New-Object System.Security.AccessControl.FileSystemAccessRule ($user, "ReadAndExecute", "Allow")

            $acl = Get-Acl $sitePath
            $acl.SetAccessRule($accessRule)
            $acl | Set-Acl $sitePath

            HP-Write-Host "Web site $($siteName) has been successfully started" -ForegroundColor Green
        }
    }
    return ($sitePath, $appPoolName)
}

function SetupDatabase() {
    $sqlServer = GetParameter "Sql Server" "Please enter SQL server name (it can be SERVER, SERVER\INSTANCE or SERVER\INSTANCE,PORT)" $overrideSavedParameters "localhost"
    $sqlDatabase = GetParameter "Sql Database" "Please enter database name" $overrideSavedParameters "Hwe2013StateStorage"

    $ErrorActionPreference = "Continue"
    HP-Write-Host ( "Trying to connect to database $($sqlDatabase) on $($sqlServer)... " ) -ForegroundColor Yellow 
    & "$PSScriptRoot\Resources\Sql\SimpleSqlExec.exe" -S "$sqlServer" -d "$sqlDatabase" -i "$PSScriptRoot\Resources\Sql\StateStore.sql" | Out-File "$PSScriptRoot/Install.log" -Append
    if ($lastexitcode -ne 0) {
        HP-Write-Host $Error[0] -ForegroundColor Red
        exit
    }
    $ErrorActionPreference = "Stop"
    HP-Write-Host 'Successfull connected to database' -ForegroundColor Green

    return "Data Source=" + $sqlServer + ";Initial Catalog=" + $sqlDatabase + ";persist security info=True;Integrated Security=SSPI"
}

function SetupHWESite($siteName, $sitePath, $sqlConnStr, $certificate) {
    HP-Write-Host ( "Deploying web site ..." ) -ForegroundColor Yellow

    $paramFile = $PSScriptRoot + "/Resources/HWE2013AppWeb.SetParameters.xml"

    Clear-Content $paramFile
    Add-Content $paramFile '<?xml version="1.0" encoding="utf-8"?>'
    Add-Content $paramFile '<parameters>'
    Add-Content $paramFile ('<setParameter name="IIS Web Application Name" value="' + $siteName + '" />')
    Add-Content $paramFile ('<setParameter name="SQLConnectionString-Web.config Connection String" value="' + $sqlConnStr + '" />')
    Add-Content $paramFile '</parameters>'
    
    $ErrorActionPreference = "Continue"
    cmd.exe  /C "$PSScriptRoot\Resources\HWE2013AppWeb.deploy.cmd /y /A:NTLM" | Out-File "$PSScriptRoot/Install.log" -Append
    if ($LASTEXITCODE -ne 0) {
        HP-Write-Host $Error[0] -ForegroundColor Red
        exit
    }
    $ErrorActionPreference = "Stop"

    HP-Write-Host ( "Modifing web.config ..." ) -ForegroundColor Yellow
    $serialNumber = $certificate.SerialNumber
    $doc = New-Object System.Xml.XmlDocument
    $doc.Load("$sitePath/Web.config")
    $node = $doc.SelectSingleNode('configuration/appSettings/add[@key="' + "ClientSigningCertificateSerialNumber" + '"]')
    $node.Attributes['value'].Value = $serialNumber
    $doc.Save("$sitePath/Web.config")

    HP-Write-Host ( "HarePoint Workflow Extensions IIS Web successfully installed" ) -ForegroundColor Green
}

function SetupTrustAndRegisterAppPrincipal($certificate) {
    $certLocation = 'cert:\localmachine\my'

    HP-Write-Host ( "Setting up trust and registering app principal..." ) -ForegroundColor Yellow    

    HP-Write-Host ( "Connecting to Central Administration web..." ) -ForegroundColor Yellow
    $site = Get-SPWebApplication -includecentraladministration | Where-Object { $_.IsAdministrationWebApplication } | Get-SPSite
    $spweb = $site[0].RootWeb

    HP-Write-Host ( "Exporting .cer certificate..." ) -ForegroundColor Yellow
    $certPath2 = $PSScriptRoot + '\Resources\HWE2013Cert.cer'

    $cerCert = Export-Certificate -Cert "$certLocation\$($certificate.Thumbprint)" -FilePath $certPath2
    
    $appName = "HarePoint Workflow Extensions 2013"
    $stsName = $appName + " STS"
    $clientId = '3805421c-b0b1-4805-902f-6349352539c6'
    $issuerId = '25e2cd43-d4a4-4db6-9fa4-13fa53cc4429'

    $realm = Get-SPAuthenticationRealm -ServiceContext $spweb.Site
    $cert = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2($certPath2)
    $currentTRA = Get-SPTrustedRootAuthority | Where-Object { $_.Name -eq ($stsName + " cert") }
    if ($currentTRA) {
        HP-Write-Host ( "Removing existing $($stsName) cert trusted root authority " ) -ForegroundColor Yellow
        $currentTRA | Remove-SPTrustedRootAuthority -Confirm:$false
        HP-Write-Host ( "$($stsName) cert trusted root authority successully removed " ) -ForegroundColor Green
    }
    HP-Write-Host ( "Creating new $($stsName) cert trusted root authority " ) -ForegroundColor Yellow
    $authority = New-SPTrustedRootAuthority -Name ($stsName + " cert") -Certificate $cert 
    HP-Write-Host ( "$($stsName) cert trusted root authority successfully created" ) -ForegroundColor Green

    $fullAppIdentifier = $clientId + '@' + $realm
    $fullIssuerIdentifier = $issuerId + '@' + $realm
    $currentTSTI = Get-SPTrustedSecurityTokenIssuer | Where-Object { $_.Name -eq $stsName } 
    if ($currentTSTI) {
        HP-Write-Host ( "Removing existing $($stsName) trusted security token issuer " ) -ForegroundColor Yellow
        $currentTSTI | Remove-SPTrustedSecurityTokenIssuer -Confirm:$false
        HP-Write-Host ( "$($stsName) trusted security token issuer successully removed " ) -ForegroundColor Green
    }
    HP-Write-Host ( "Creating new $($stsName) trusted security token issuer " ) -ForegroundColor Yellow
    $sts = New-SPTrustedSecurityTokenIssuer -Name $stsName -Certificate $cert -RegisteredIssuerName $fullIssuerIdentifier -IsTrustBroker
    HP-Write-Host ( "$($stsName) trusted security token issuer successfully created" ) -ForegroundColor Green

    HP-Write-Host ( "Registering app principal" ) -ForegroundColor Yellow
    $appPrincipal = Register-SPAppPrincipal -NameIdentifier $fullAppIdentifier -Site $spweb -DisplayName $appName
    HP-Write-Host ( "App principal has been successfully registered" ) -ForegroundColor Green
}

function DeploySolution() {
    $wspName = "hwe2013.wsp"
    $wspFullPath = $PSScriptRoot + '\Resources\' + $wspName
  
    $installedSolution = Get-SPSolution | Where-Object Name -eq $wspName
    try {     
        if ($null -ne $installedSolution) {
            if ($installedSolution.Deployed) {
                WaitForTimer($wspName)  
                Uninstall-SPSolution $wspName -confirm:$false
                WaitForTimer($wspName)   
            }
            HP-Write-Host "Removing solution from the farm" -ForegroundColor Yellow 
            Remove-SPSolution $wspName -Confirm:$false
            Start-Sleep 3
        }
  
        WaitForTimer($wspName) 
        $solution = Add-SPSolution -LiteralPath "$wspFullPath"
        Install-SPSolution -Identity $wspName -GACDeployment:$true -Force:$true
        WaitForTimer($wspName)    
  
        HP-Write-Host "Successfully deployed the package" -ForegroundColor Green 
    } 
    catch {
        HP-Write-Host "Exception occured on deploy : " $Error[0].Exception.Message -ForegroundColor Red  
    }
}

function CheckInstallation() {
    HP-Write-Host ("Specify web site url: ") -NoNewLine
    $spWebUrl = Read-Host
    $spWeb = Get-SPWeb $spWebUrl
    $spSite = $spWeb.Site
    $spWebApp = $spSite.WebApplication

    HP-Write-Host "Check features..." -ForegroundColor Yellow

    $feature = Get-SPFeature -Identity "adb79bf3-9e32-4516-90d1-9fa5dbd4bbf8" -Web $spWeb -ErrorAction SilentlyContinue
    if (($null -eq ($feature))) {
        HP-Write-Host "HarePoint Workflow Extensions 2013 Activation..." -ForegroundColor Yellow
        Enable-SPFeature -Identity adb79bf3-9e32-4516-90d1-9fa5dbd4bbf8 -Url $spWeb.Url -ErrorAction Stop
        HP-Write-Host "HarePoint Workflow Extensions 2013 Activation activated" -ForegroundColor Yellow
    }

    $feature = Get-SPFeature -Identity "72163223-4a76-4fdf-9de7-8a3b8eaa9a13" -Web $spWeb -ErrorAction SilentlyContinue
    HP-Write-Host "HarePoint Workflow Extensions 2013 Actions..." -ForegroundColor Yellow
    if (($null -ne ($feature))) {
        Disable-SPFeature -Identity 72163223-4a76-4fdf-9de7-8a3b8eaa9a13 -Url $spWeb.Url -ErrorAction Stop
        HP-Write-Host "HarePoint Workflow Extensions 2013 Actions deactivated" -ForegroundColor Yellow
    }
	$feature = Get-SPFeature -Identity "72163223-4a76-4fdf-9de7-8a3b8eaa9a13" -Web $spWeb -ErrorAction SilentlyContinue
    if (($null -eq ($feature))) {
        Enable-SPFeature -Identity 72163223-4a76-4fdf-9de7-8a3b8eaa9a13 -Url $spWeb.Url -ErrorAction Stop
        HP-Write-Host "HarePoint Workflow Extensions 2013 Actions activated" -ForegroundColor Yellow
    }

    $feature = Get-SPFeature -Identity "94ad430a-e8df-403c-8e95-1040405f983b" -WebApplication $spWebApp -ErrorAction SilentlyContinue
    if (($null -eq ($feature))) {
        HP-Write-Host "HarePoint Workflow Extensions 2013..."
        Enable-SPFeature -Identity 94ad430a-e8df-403c-8e95-1040405f983b -Url $spWebApp.Url -ErrorAction Stop
        HP-Write-Host "HarePoint Workflow Extensions 2013 activated"
    }
        
    HP-Write-Host "Features activated" -ForegroundColor Green

    $httpPort = GetParameter "Http port" "Please enter a port number for a site" $true "8187"
    $defaultServer = GetFQDNForCurrentServer
    $server = ReadOptionalParameterFromConfig "Server" $defaultServer
        
    HP-Write-Host ("Validating HWE Web Application(http://$server" + ":$httpPort)") -ForegroundColor Yellow

    $encodedSpWebUrl = [System.Web.HttpUtility]::UrlEncode($spWebUrl) 

    try {
        $ErrorActionPreference = "Continue"
        $response = Invoke-WebRequest -URI ("http://$server" + ":$httpPort/api/actions/status?url=$encodedSpWebUrl") 
        HP-Write-Host "HWE Web Application validation status successful" -ForegroundColor Green
    }
    catch {
        If ($_.Exception.Response.StatusCode.value__) {
            $result = $_.Exception.Response.GetResponseStream()
            $reader = New-Object System.IO.StreamReader($result)
            $reader.BaseStream.Position = 0
            $reader.DiscardBufferedData()
            $responseBody = $reader.ReadToEnd()
            if ($responseBody.StartsWith('{')) {
                $responseBody = $responseBody | ConvertFrom-Json
            }

            if ($responseBody.Message) {
                HP-Write-Host $responseBody.Message -ForegroundColor Red
                exit
            }             
        }
        HP-Write-Host "HWE Web Application validation status failed. $_" -ForegroundColor Red
    }
    finally {
        $ErrorActionPreference = "Stop"
    }
}

function Run() {
    if (-Not(IsNetFrameworkInstalled)) {
        HP-Write-Host "This application requires .NET Framework 4.6.1. Please install the .NET Framework then run this installer again."
        exit
    }

    if (-Not(IsMSDeployInstalled)) {
        HP-Write-Host "This application requires Web Deploy for IIS. Please install Web Deploy then run this installer again (https://www.iis.net/downloads/microsoft/web-deploy)."
        exit
    }

    $parametersExist = DisplaySavedParameters

    if ($parametersExist) {
        HP-Write-Host ("Run a script with these parameters? [Y/n]") -NoNewLine
        $line = Read-Host
        $overrideSavedParameters = ($null -eq $line) -or ($line -eq "") -or ($line.ToString().ToLower() -eq "y")
    }

    HP-Write-Host "Specify installation mode:"
    HP-Write-Host "1. Full"
    HP-Write-Host "2. Workflow actions IIS web site"
    HP-Write-Host "3. SharePoint package with action definitions and management pages"

    $mode = GetParameter "Installation mode" "Enter mode" $overrideSavedParameters "1"

    $defaultServer = GetFQDNForCurrentServer
    $server = ReadOptionalParameterFromConfig "Server" $defaultServer

    if ($mode -ne "4") {
        $certificate = CreateAndImportCertificate
    }

    if (($mode -eq "1") -Or ($mode -eq "2")) {
        $siteName = GetParameter "Site name" "Please enter site name" $overrideSavedParameters "Hwe2013"

        ($sitePath, $appPoolName) = CreateHWESiteIfNoExist $siteName
 
        $sqlConnStr = SetupDatabase
        SetupHWESite $siteName $sitePath $sqlConnStr $certificate

        if (mode -eq "1") {
            try {
                Add-PSSnapin Microsoft.SharePoint.PowerShell
                $httpPort = GetParameter "Http port" "Please enter a port number for a site" $overrideSavedParameters "8187"
                ConfigureFarmPropertyBag "HWE2013_AppUrl" ("http://$server" + ":$httpPort")
            }
            catch {
                HP-Write-Host "Warning: cannot configure farm property bag."
            }
        }
    }

    if (($mode -eq "1") -or ($mode -eq "3")) {
        Add-PSSnapin Microsoft.SharePoint.PowerShell    
        SetupTrustAndRegisterAppPrincipal  $certificate
        DeploySolution    
    }

    if (($mode -eq "4") -or ($mode -eq "1") -or ($mode -eq "3")) {
        HP-Write-Host ("Do you want to validate installation and activate HarePoint Workflow Extensions on specified web site? [Y/n]")
        Add-PSSnapin Microsoft.SharePoint.PowerShell

        $line = Read-Host 
        $validate = ($null -eq $line) -or ($line -eq "") -or ($line.ToString().ToLower() -eq "y")

        if ($validate) {
            CheckInstallation
        }
    }

    HP-Write-Host "Press any key to continue..."
    $x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
}

$ErrorActionPreference = "Stop"
try {
    Run 
}
catch {
    HP-Write-Host "Error occured. $_" -ForegroundColor Red
}

